"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tendermint.p2p.types_pb2
import tendermint.types.block_pb2
import tendermint.types.types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class GetValidatorSetByHeightRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HEIGHT_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    height: builtins.int = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        height : builtins.int = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"height",b"height",u"pagination",b"pagination"]) -> None: ...
global___GetValidatorSetByHeightRequest = GetValidatorSetByHeightRequest

class GetValidatorSetByHeightResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    block_height: builtins.int = ...

    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Validator]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        block_height : builtins.int = ...,
        validators : typing.Optional[typing.Iterable[global___Validator]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"block_height",b"block_height",u"pagination",b"pagination",u"validators",b"validators"]) -> None: ...
global___GetValidatorSetByHeightResponse = GetValidatorSetByHeightResponse

class GetLatestValidatorSetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> None: ...
global___GetLatestValidatorSetRequest = GetLatestValidatorSetRequest

class GetLatestValidatorSetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    block_height: builtins.int = ...

    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Validator]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        block_height : builtins.int = ...,
        validators : typing.Optional[typing.Iterable[global___Validator]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"block_height",b"block_height",u"pagination",b"pagination",u"validators",b"validators"]) -> None: ...
global___GetLatestValidatorSetResponse = GetLatestValidatorSetResponse

class Validator(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ADDRESS_FIELD_NUMBER: builtins.int
    PUB_KEY_FIELD_NUMBER: builtins.int
    VOTING_POWER_FIELD_NUMBER: builtins.int
    PROPOSER_PRIORITY_FIELD_NUMBER: builtins.int
    address: typing.Text = ...
    voting_power: builtins.int = ...
    proposer_priority: builtins.int = ...

    @property
    def pub_key(self) -> google.protobuf.any_pb2.Any: ...

    def __init__(self,
        *,
        address : typing.Text = ...,
        pub_key : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        voting_power : builtins.int = ...,
        proposer_priority : builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pub_key",b"pub_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"address",b"address",u"proposer_priority",b"proposer_priority",u"pub_key",b"pub_key",u"voting_power",b"voting_power"]) -> None: ...
global___Validator = Validator

class GetBlockByHeightRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int = ...

    def __init__(self,
        *,
        height : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"height",b"height"]) -> None: ...
global___GetBlockByHeightRequest = GetBlockByHeightRequest

class GetBlockByHeightResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    BLOCK_ID_FIELD_NUMBER: builtins.int
    BLOCK_FIELD_NUMBER: builtins.int

    @property
    def block_id(self) -> tendermint.types.types_pb2.BlockID: ...

    @property
    def block(self) -> tendermint.types.block_pb2.Block: ...

    def __init__(self,
        *,
        block_id : typing.Optional[tendermint.types.types_pb2.BlockID] = ...,
        block : typing.Optional[tendermint.types.block_pb2.Block] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"block",b"block",u"block_id",b"block_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"block",b"block",u"block_id",b"block_id"]) -> None: ...
global___GetBlockByHeightResponse = GetBlockByHeightResponse

class GetLatestBlockRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___GetLatestBlockRequest = GetLatestBlockRequest

class GetLatestBlockResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    BLOCK_ID_FIELD_NUMBER: builtins.int
    BLOCK_FIELD_NUMBER: builtins.int

    @property
    def block_id(self) -> tendermint.types.types_pb2.BlockID: ...

    @property
    def block(self) -> tendermint.types.block_pb2.Block: ...

    def __init__(self,
        *,
        block_id : typing.Optional[tendermint.types.types_pb2.BlockID] = ...,
        block : typing.Optional[tendermint.types.block_pb2.Block] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"block",b"block",u"block_id",b"block_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"block",b"block",u"block_id",b"block_id"]) -> None: ...
global___GetLatestBlockResponse = GetLatestBlockResponse

class GetSyncingRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___GetSyncingRequest = GetSyncingRequest

class GetSyncingResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    SYNCING_FIELD_NUMBER: builtins.int
    syncing: builtins.bool = ...

    def __init__(self,
        *,
        syncing : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"syncing",b"syncing"]) -> None: ...
global___GetSyncingResponse = GetSyncingResponse

class GetNodeInfoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___GetNodeInfoRequest = GetNodeInfoRequest

class GetNodeInfoResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DEFAULT_NODE_INFO_FIELD_NUMBER: builtins.int
    APPLICATION_VERSION_FIELD_NUMBER: builtins.int

    @property
    def default_node_info(self) -> tendermint.p2p.types_pb2.DefaultNodeInfo: ...

    @property
    def application_version(self) -> global___VersionInfo: ...

    def __init__(self,
        *,
        default_node_info : typing.Optional[tendermint.p2p.types_pb2.DefaultNodeInfo] = ...,
        application_version : typing.Optional[global___VersionInfo] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"application_version",b"application_version",u"default_node_info",b"default_node_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"application_version",b"application_version",u"default_node_info",b"default_node_info"]) -> None: ...
global___GetNodeInfoResponse = GetNodeInfoResponse

class VersionInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NAME_FIELD_NUMBER: builtins.int
    APP_NAME_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    GIT_COMMIT_FIELD_NUMBER: builtins.int
    BUILD_TAGS_FIELD_NUMBER: builtins.int
    GO_VERSION_FIELD_NUMBER: builtins.int
    BUILD_DEPS_FIELD_NUMBER: builtins.int
    COSMOS_SDK_VERSION_FIELD_NUMBER: builtins.int
    name: typing.Text = ...
    app_name: typing.Text = ...
    version: typing.Text = ...
    git_commit: typing.Text = ...
    build_tags: typing.Text = ...
    go_version: typing.Text = ...
    cosmos_sdk_version: typing.Text = ...

    @property
    def build_deps(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Module]: ...

    def __init__(self,
        *,
        name : typing.Text = ...,
        app_name : typing.Text = ...,
        version : typing.Text = ...,
        git_commit : typing.Text = ...,
        build_tags : typing.Text = ...,
        go_version : typing.Text = ...,
        build_deps : typing.Optional[typing.Iterable[global___Module]] = ...,
        cosmos_sdk_version : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"app_name",b"app_name",u"build_deps",b"build_deps",u"build_tags",b"build_tags",u"cosmos_sdk_version",b"cosmos_sdk_version",u"git_commit",b"git_commit",u"go_version",b"go_version",u"name",b"name",u"version",b"version"]) -> None: ...
global___VersionInfo = VersionInfo

class Module(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PATH_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    path: typing.Text = ...
    version: typing.Text = ...
    sum: typing.Text = ...

    def __init__(self,
        *,
        path : typing.Text = ...,
        version : typing.Text = ...,
        sum : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"path",b"path",u"sum",b"sum",u"version",b"version"]) -> None: ...
global___Module = Module



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.staking.v1beta1.staking_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class QueryValidatorsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    STATUS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    status: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        status : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"status",b"status"]) -> None: ...
global___QueryValidatorsRequest = QueryValidatorsRequest

class QueryValidatorsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.Validator]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        validators : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.Validator]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"validators",b"validators"]) -> None: ...
global___QueryValidatorsResponse = QueryValidatorsResponse

class QueryValidatorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text = ...

    def __init__(self,
        *,
        validator_addr : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorRequest = QueryValidatorRequest

class QueryValidatorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_FIELD_NUMBER: builtins.int

    @property
    def validator(self) -> cosmos.staking.v1beta1.staking_pb2.Validator: ...

    def __init__(self,
        *,
        validator : typing.Optional[cosmos.staking.v1beta1.staking_pb2.Validator] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"validator",b"validator"]) -> None: ...
global___QueryValidatorResponse = QueryValidatorResponse

class QueryValidatorDelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        validator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorDelegationsRequest = QueryValidatorDelegationsRequest

class QueryValidatorDelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def delegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        delegation_responses : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegation_responses",b"delegation_responses",u"pagination",b"pagination"]) -> None: ...
global___QueryValidatorDelegationsResponse = QueryValidatorDelegationsResponse

class QueryValidatorUnbondingDelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        validator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryValidatorUnbondingDelegationsRequest = QueryValidatorUnbondingDelegationsRequest

class QueryValidatorUnbondingDelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    UNBONDING_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def unbonding_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        unbonding_responses : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"unbonding_responses",b"unbonding_responses"]) -> None: ...
global___QueryValidatorUnbondingDelegationsResponse = QueryValidatorUnbondingDelegationsResponse

class QueryDelegationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...
    validator_addr: typing.Text = ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        validator_addr : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryDelegationRequest = QueryDelegationRequest

class QueryDelegationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATION_RESPONSE_FIELD_NUMBER: builtins.int

    @property
    def delegation_response(self) -> cosmos.staking.v1beta1.staking_pb2.DelegationResponse: ...

    def __init__(self,
        *,
        delegation_response : typing.Optional[cosmos.staking.v1beta1.staking_pb2.DelegationResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"delegation_response",b"delegation_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegation_response",b"delegation_response"]) -> None: ...
global___QueryDelegationResponse = QueryDelegationResponse

class QueryUnbondingDelegationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...
    validator_addr: typing.Text = ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        validator_addr : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryUnbondingDelegationRequest = QueryUnbondingDelegationRequest

class QueryUnbondingDelegationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    UNBOND_FIELD_NUMBER: builtins.int

    @property
    def unbond(self) -> cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation: ...

    def __init__(self,
        *,
        unbond : typing.Optional[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"unbond",b"unbond"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"unbond",b"unbond"]) -> None: ...
global___QueryUnbondingDelegationResponse = QueryUnbondingDelegationResponse

class QueryDelegatorDelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"pagination",b"pagination"]) -> None: ...
global___QueryDelegatorDelegationsRequest = QueryDelegatorDelegationsRequest

class QueryDelegatorDelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def delegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        delegation_responses : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.DelegationResponse]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegation_responses",b"delegation_responses",u"pagination",b"pagination"]) -> None: ...
global___QueryDelegatorDelegationsResponse = QueryDelegatorDelegationsResponse

class QueryDelegatorUnbondingDelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"pagination",b"pagination"]) -> None: ...
global___QueryDelegatorUnbondingDelegationsRequest = QueryDelegatorUnbondingDelegationsRequest

class QueryDelegatorUnbondingDelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    UNBONDING_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def unbonding_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        unbonding_responses : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.UnbondingDelegation]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"unbonding_responses",b"unbonding_responses"]) -> None: ...
global___QueryDelegatorUnbondingDelegationsResponse = QueryDelegatorUnbondingDelegationsResponse

class QueryRedelegationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    SRC_VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    DST_VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...
    src_validator_addr: typing.Text = ...
    dst_validator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        src_validator_addr : typing.Text = ...,
        dst_validator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"dst_validator_addr",b"dst_validator_addr",u"pagination",b"pagination",u"src_validator_addr",b"src_validator_addr"]) -> None: ...
global___QueryRedelegationsRequest = QueryRedelegationsRequest

class QueryRedelegationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    REDELEGATION_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def redelegation_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.RedelegationResponse]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        redelegation_responses : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.RedelegationResponse]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"redelegation_responses",b"redelegation_responses"]) -> None: ...
global___QueryRedelegationsResponse = QueryRedelegationsResponse

class QueryDelegatorValidatorsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"pagination",b"pagination"]) -> None: ...
global___QueryDelegatorValidatorsRequest = QueryDelegatorValidatorsRequest

class QueryDelegatorValidatorsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATORS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.staking.v1beta1.staking_pb2.Validator]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        validators : typing.Optional[typing.Iterable[cosmos.staking.v1beta1.staking_pb2.Validator]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"validators",b"validators"]) -> None: ...
global___QueryDelegatorValidatorsResponse = QueryDelegatorValidatorsResponse

class QueryDelegatorValidatorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDR_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    delegator_addr: typing.Text = ...
    validator_addr: typing.Text = ...

    def __init__(self,
        *,
        delegator_addr : typing.Text = ...,
        validator_addr : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_addr",b"delegator_addr",u"validator_addr",b"validator_addr"]) -> None: ...
global___QueryDelegatorValidatorRequest = QueryDelegatorValidatorRequest

class QueryDelegatorValidatorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_FIELD_NUMBER: builtins.int

    @property
    def validator(self) -> cosmos.staking.v1beta1.staking_pb2.Validator: ...

    def __init__(self,
        *,
        validator : typing.Optional[cosmos.staking.v1beta1.staking_pb2.Validator] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"validator",b"validator"]) -> None: ...
global___QueryDelegatorValidatorResponse = QueryDelegatorValidatorResponse

class QueryHistoricalInfoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int = ...

    def __init__(self,
        *,
        height : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"height",b"height"]) -> None: ...
global___QueryHistoricalInfoRequest = QueryHistoricalInfoRequest

class QueryHistoricalInfoResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HIST_FIELD_NUMBER: builtins.int

    @property
    def hist(self) -> cosmos.staking.v1beta1.staking_pb2.HistoricalInfo: ...

    def __init__(self,
        *,
        hist : typing.Optional[cosmos.staking.v1beta1.staking_pb2.HistoricalInfo] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"hist",b"hist"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"hist",b"hist"]) -> None: ...
global___QueryHistoricalInfoResponse = QueryHistoricalInfoResponse

class QueryPoolRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___QueryPoolRequest = QueryPoolRequest

class QueryPoolResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    POOL_FIELD_NUMBER: builtins.int

    @property
    def pool(self) -> cosmos.staking.v1beta1.staking_pb2.Pool: ...

    def __init__(self,
        *,
        pool : typing.Optional[cosmos.staking.v1beta1.staking_pb2.Pool] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pool",b"pool"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pool",b"pool"]) -> None: ...
global___QueryPoolResponse = QueryPoolResponse

class QueryParamsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PARAMS_FIELD_NUMBER: builtins.int

    @property
    def params(self) -> cosmos.staking.v1beta1.staking_pb2.Params: ...

    def __init__(self,
        *,
        params : typing.Optional[cosmos.staking.v1beta1.staking_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from cosmos.tx.v1beta1 import service_pb2 as cosmos_dot_tx_dot_v1beta1_dot_service__pb2


class ServiceStub(object):
    """Service defines a gRPC service for interacting with transactions.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Simulate = channel.unary_unary(
                '/cosmos.tx.v1beta1.Service/Simulate',
                request_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateRequest.SerializeToString,
                response_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateResponse.FromString,
                )
        self.GetTx = channel.unary_unary(
                '/cosmos.tx.v1beta1.Service/GetTx',
                request_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxRequest.SerializeToString,
                response_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxResponse.FromString,
                )
        self.BroadcastTx = channel.unary_unary(
                '/cosmos.tx.v1beta1.Service/BroadcastTx',
                request_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxRequest.SerializeToString,
                response_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxResponse.FromString,
                )
        self.GetTxsEvent = channel.unary_unary(
                '/cosmos.tx.v1beta1.Service/GetTxsEvent',
                request_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventRequest.SerializeToString,
                response_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventResponse.FromString,
                )


class ServiceServicer(object):
    """Service defines a gRPC service for interacting with transactions.
    """

    def Simulate(self, request, context):
        """Simulate simulates executing a transaction for estimating gas usage.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTx(self, request, context):
        """GetTx fetches a tx by hash.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BroadcastTx(self, request, context):
        """BroadcastTx broadcast transaction.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTxsEvent(self, request, context):
        """GetTxsEvent fetches txs by event.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Simulate': grpc.unary_unary_rpc_method_handler(
                    servicer.Simulate,
                    request_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateRequest.FromString,
                    response_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateResponse.SerializeToString,
            ),
            'GetTx': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTx,
                    request_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxRequest.FromString,
                    response_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxResponse.SerializeToString,
            ),
            'BroadcastTx': grpc.unary_unary_rpc_method_handler(
                    servicer.BroadcastTx,
                    request_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxRequest.FromString,
                    response_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxResponse.SerializeToString,
            ),
            'GetTxsEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTxsEvent,
                    request_deserializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventRequest.FromString,
                    response_serializer=cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cosmos.tx.v1beta1.Service', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Service(object):
    """Service defines a gRPC service for interacting with transactions.
    """

    @staticmethod
    def Simulate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.tx.v1beta1.Service/Simulate',
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateRequest.SerializeToString,
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.SimulateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.tx.v1beta1.Service/GetTx',
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxRequest.SerializeToString,
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BroadcastTx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.tx.v1beta1.Service/BroadcastTx',
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxRequest.SerializeToString,
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.BroadcastTxResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTxsEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.tx.v1beta1.Service/GetTxsEvent',
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventRequest.SerializeToString,
            cosmos_dot_tx_dot_v1beta1_dot_service__pb2.GetTxsEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ibc.core.channel.v1.channel_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class GenesisState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CHANNELS_FIELD_NUMBER: builtins.int
    ACKNOWLEDGEMENTS_FIELD_NUMBER: builtins.int
    COMMITMENTS_FIELD_NUMBER: builtins.int
    RECEIPTS_FIELD_NUMBER: builtins.int
    SEND_SEQUENCES_FIELD_NUMBER: builtins.int
    RECV_SEQUENCES_FIELD_NUMBER: builtins.int
    ACK_SEQUENCES_FIELD_NUMBER: builtins.int
    NEXT_CHANNEL_SEQUENCE_FIELD_NUMBER: builtins.int
    next_channel_sequence: builtins.int = ...

    @property
    def channels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.channel.v1.channel_pb2.IdentifiedChannel]: ...

    @property
    def acknowledgements(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.channel.v1.channel_pb2.PacketState]: ...

    @property
    def commitments(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.channel.v1.channel_pb2.PacketState]: ...

    @property
    def receipts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.channel.v1.channel_pb2.PacketState]: ...

    @property
    def send_sequences(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PacketSequence]: ...

    @property
    def recv_sequences(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PacketSequence]: ...

    @property
    def ack_sequences(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PacketSequence]: ...

    def __init__(self,
        *,
        channels : typing.Optional[typing.Iterable[ibc.core.channel.v1.channel_pb2.IdentifiedChannel]] = ...,
        acknowledgements : typing.Optional[typing.Iterable[ibc.core.channel.v1.channel_pb2.PacketState]] = ...,
        commitments : typing.Optional[typing.Iterable[ibc.core.channel.v1.channel_pb2.PacketState]] = ...,
        receipts : typing.Optional[typing.Iterable[ibc.core.channel.v1.channel_pb2.PacketState]] = ...,
        send_sequences : typing.Optional[typing.Iterable[global___PacketSequence]] = ...,
        recv_sequences : typing.Optional[typing.Iterable[global___PacketSequence]] = ...,
        ack_sequences : typing.Optional[typing.Iterable[global___PacketSequence]] = ...,
        next_channel_sequence : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"ack_sequences",b"ack_sequences",u"acknowledgements",b"acknowledgements",u"channels",b"channels",u"commitments",b"commitments",u"next_channel_sequence",b"next_channel_sequence",u"receipts",b"receipts",u"recv_sequences",b"recv_sequences",u"send_sequences",b"send_sequences"]) -> None: ...
global___GenesisState = GenesisState

class PacketSequence(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    channel_id: typing.Text = ...
    sequence: builtins.int = ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel_id : typing.Text = ...,
        sequence : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel_id",b"channel_id",u"port_id",b"port_id",u"sequence",b"sequence"]) -> None: ...
global___PacketSequence = PacketSequence



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class LegacyAminoPubKey(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    THRESHOLD_FIELD_NUMBER: builtins.int
    PUBLIC_KEYS_FIELD_NUMBER: builtins.int
    threshold: builtins.int = ...

    @property
    def public_keys(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]: ...

    def __init__(self,
        *,
        threshold : builtins.int = ...,
        public_keys : typing.Optional[typing.Iterable[google.protobuf.any_pb2.Any]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"public_keys",b"public_keys",u"threshold",b"threshold"]) -> None: ...
global___LegacyAminoPubKey = LegacyAminoPubKey



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.abci.v1beta1.abci_pb2
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.tx.v1beta1.tx_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class OrderBy(metaclass=_OrderBy):
    V = typing.NewType('V', builtins.int)

global___OrderBy = OrderBy

ORDER_BY_UNSPECIFIED = OrderBy.V(0)
ORDER_BY_ASC = OrderBy.V(1)
ORDER_BY_DESC = OrderBy.V(2)

class _OrderBy(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[OrderBy.V], builtins.type):  # type: ignore
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    ORDER_BY_UNSPECIFIED = OrderBy.V(0)
    ORDER_BY_ASC = OrderBy.V(1)
    ORDER_BY_DESC = OrderBy.V(2)

class BroadcastMode(metaclass=_BroadcastMode):
    V = typing.NewType('V', builtins.int)

global___BroadcastMode = BroadcastMode

BROADCAST_MODE_UNSPECIFIED = BroadcastMode.V(0)
BROADCAST_MODE_BLOCK = BroadcastMode.V(1)
BROADCAST_MODE_SYNC = BroadcastMode.V(2)
BROADCAST_MODE_ASYNC = BroadcastMode.V(3)

class _BroadcastMode(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BroadcastMode.V], builtins.type):  # type: ignore
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    BROADCAST_MODE_UNSPECIFIED = BroadcastMode.V(0)
    BROADCAST_MODE_BLOCK = BroadcastMode.V(1)
    BROADCAST_MODE_SYNC = BroadcastMode.V(2)
    BROADCAST_MODE_ASYNC = BroadcastMode.V(3)

class GetTxsEventRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EVENTS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    ORDER_BY_FIELD_NUMBER: builtins.int

    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    order_by: global___OrderBy.V = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        events : typing.Optional[typing.Iterable[typing.Text]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        order_by : global___OrderBy.V = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"events",b"events",u"order_by",b"order_by",u"pagination",b"pagination"]) -> None: ...
global___GetTxsEventRequest = GetTxsEventRequest

class GetTxsEventResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TXS_FIELD_NUMBER: builtins.int
    TX_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def txs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.tx.v1beta1.tx_pb2.Tx]: ...

    @property
    def tx_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.abci.v1beta1.abci_pb2.TxResponse]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        txs : typing.Optional[typing.Iterable[cosmos.tx.v1beta1.tx_pb2.Tx]] = ...,
        tx_responses : typing.Optional[typing.Iterable[cosmos.base.abci.v1beta1.abci_pb2.TxResponse]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination",u"tx_responses",b"tx_responses",u"txs",b"txs"]) -> None: ...
global___GetTxsEventResponse = GetTxsEventResponse

class BroadcastTxRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TX_BYTES_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    tx_bytes: builtins.bytes = ...
    mode: global___BroadcastMode.V = ...

    def __init__(self,
        *,
        tx_bytes : builtins.bytes = ...,
        mode : global___BroadcastMode.V = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"mode",b"mode",u"tx_bytes",b"tx_bytes"]) -> None: ...
global___BroadcastTxRequest = BroadcastTxRequest

class BroadcastTxResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TX_RESPONSE_FIELD_NUMBER: builtins.int

    @property
    def tx_response(self) -> cosmos.base.abci.v1beta1.abci_pb2.TxResponse: ...

    def __init__(self,
        *,
        tx_response : typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.TxResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"tx_response",b"tx_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"tx_response",b"tx_response"]) -> None: ...
global___BroadcastTxResponse = BroadcastTxResponse

class SimulateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TX_FIELD_NUMBER: builtins.int

    @property
    def tx(self) -> cosmos.tx.v1beta1.tx_pb2.Tx: ...

    def __init__(self,
        *,
        tx : typing.Optional[cosmos.tx.v1beta1.tx_pb2.Tx] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"tx",b"tx"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"tx",b"tx"]) -> None: ...
global___SimulateRequest = SimulateRequest

class SimulateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    GAS_INFO_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int

    @property
    def gas_info(self) -> cosmos.base.abci.v1beta1.abci_pb2.GasInfo: ...

    @property
    def result(self) -> cosmos.base.abci.v1beta1.abci_pb2.Result: ...

    def __init__(self,
        *,
        gas_info : typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.GasInfo] = ...,
        result : typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.Result] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"gas_info",b"gas_info",u"result",b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"gas_info",b"gas_info",u"result",b"result"]) -> None: ...
global___SimulateResponse = SimulateResponse

class GetTxRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HASH_FIELD_NUMBER: builtins.int
    hash: typing.Text = ...

    def __init__(self,
        *,
        hash : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"hash",b"hash"]) -> None: ...
global___GetTxRequest = GetTxRequest

class GetTxResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TX_FIELD_NUMBER: builtins.int
    TX_RESPONSE_FIELD_NUMBER: builtins.int

    @property
    def tx(self) -> cosmos.tx.v1beta1.tx_pb2.Tx: ...

    @property
    def tx_response(self) -> cosmos.base.abci.v1beta1.abci_pb2.TxResponse: ...

    def __init__(self,
        *,
        tx : typing.Optional[cosmos.tx.v1beta1.tx_pb2.Tx] = ...,
        tx_response : typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.TxResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"tx",b"tx",u"tx_response",b"tx_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"tx",b"tx",u"tx_response",b"tx_response"]) -> None: ...
global___GetTxResponse = GetTxResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc

from .service_pb2 import *
class ServiceStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    Simulate:grpc.UnaryUnaryMultiCallable[
        global___SimulateRequest,
        global___SimulateResponse] = ...

    GetTx:grpc.UnaryUnaryMultiCallable[
        global___GetTxRequest,
        global___GetTxResponse] = ...

    BroadcastTx:grpc.UnaryUnaryMultiCallable[
        global___BroadcastTxRequest,
        global___BroadcastTxResponse] = ...

    GetTxsEvent:grpc.UnaryUnaryMultiCallable[
        global___GetTxsEventRequest,
        global___GetTxsEventResponse] = ...


class ServiceServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def Simulate(self,
        request: global___SimulateRequest,
        context: grpc.ServicerContext,
    ) -> global___SimulateResponse: ...

    @abc.abstractmethod
    def GetTx(self,
        request: global___GetTxRequest,
        context: grpc.ServicerContext,
    ) -> global___GetTxResponse: ...

    @abc.abstractmethod
    def BroadcastTx(self,
        request: global___BroadcastTxRequest,
        context: grpc.ServicerContext,
    ) -> global___BroadcastTxResponse: ...

    @abc.abstractmethod
    def GetTxsEvent(self,
        request: global___GetTxsEventRequest,
        context: grpc.ServicerContext,
    ) -> global___GetTxsEventResponse: ...


def add_ServiceServicer_to_server(servicer: ServiceServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class MsgSetWithdrawAddress(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    WITHDRAW_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    withdraw_address: typing.Text = ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        withdraw_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"withdraw_address",b"withdraw_address"]) -> None: ...
global___MsgSetWithdrawAddress = MsgSetWithdrawAddress

class MsgSetWithdrawAddressResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgSetWithdrawAddressResponse = MsgSetWithdrawAddressResponse

class MsgWithdrawDelegatorReward(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_address: typing.Text = ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"validator_address",b"validator_address"]) -> None: ...
global___MsgWithdrawDelegatorReward = MsgWithdrawDelegatorReward

class MsgWithdrawDelegatorRewardResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgWithdrawDelegatorRewardResponse = MsgWithdrawDelegatorRewardResponse

class MsgWithdrawValidatorCommission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    validator_address: typing.Text = ...

    def __init__(self,
        *,
        validator_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"validator_address",b"validator_address"]) -> None: ...
global___MsgWithdrawValidatorCommission = MsgWithdrawValidatorCommission

class MsgWithdrawValidatorCommissionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgWithdrawValidatorCommissionResponse = MsgWithdrawValidatorCommissionResponse

class MsgFundCommunityPool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    AMOUNT_FIELD_NUMBER: builtins.int
    DEPOSITOR_FIELD_NUMBER: builtins.int
    depositor: typing.Text = ...

    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...

    def __init__(self,
        *,
        amount : typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        depositor : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"amount",b"amount",u"depositor",b"depositor"]) -> None: ...
global___MsgFundCommunityPool = MsgFundCommunityPool

class MsgFundCommunityPoolResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgFundCommunityPoolResponse = MsgFundCommunityPoolResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import ibc.core.channel.v1.channel_pb2
import ibc.core.client.v1.client_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class MsgChannelOpenInit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    signer: typing.Text = ...

    @property
    def channel(self) -> ibc.core.channel.v1.channel_pb2.Channel: ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel : typing.Optional[ibc.core.channel.v1.channel_pb2.Channel] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"channel",b"channel"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel",b"channel",u"port_id",b"port_id",u"signer",b"signer"]) -> None: ...
global___MsgChannelOpenInit = MsgChannelOpenInit

class MsgChannelOpenInitResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelOpenInitResponse = MsgChannelOpenInitResponse

class MsgChannelOpenTry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    PREVIOUS_CHANNEL_ID_FIELD_NUMBER: builtins.int
    CHANNEL_FIELD_NUMBER: builtins.int
    COUNTERPARTY_VERSION_FIELD_NUMBER: builtins.int
    PROOF_INIT_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    previous_channel_id: typing.Text = ...
    counterparty_version: typing.Text = ...
    proof_init: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def channel(self) -> ibc.core.channel.v1.channel_pb2.Channel: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        previous_channel_id : typing.Text = ...,
        channel : typing.Optional[ibc.core.channel.v1.channel_pb2.Channel] = ...,
        counterparty_version : typing.Text = ...,
        proof_init : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"channel",b"channel",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel",b"channel",u"counterparty_version",b"counterparty_version",u"port_id",b"port_id",u"previous_channel_id",b"previous_channel_id",u"proof_height",b"proof_height",u"proof_init",b"proof_init",u"signer",b"signer"]) -> None: ...
global___MsgChannelOpenTry = MsgChannelOpenTry

class MsgChannelOpenTryResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelOpenTryResponse = MsgChannelOpenTryResponse

class MsgChannelOpenAck(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    COUNTERPARTY_CHANNEL_ID_FIELD_NUMBER: builtins.int
    COUNTERPARTY_VERSION_FIELD_NUMBER: builtins.int
    PROOF_TRY_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    channel_id: typing.Text = ...
    counterparty_channel_id: typing.Text = ...
    counterparty_version: typing.Text = ...
    proof_try: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel_id : typing.Text = ...,
        counterparty_channel_id : typing.Text = ...,
        counterparty_version : typing.Text = ...,
        proof_try : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel_id",b"channel_id",u"counterparty_channel_id",b"counterparty_channel_id",u"counterparty_version",b"counterparty_version",u"port_id",b"port_id",u"proof_height",b"proof_height",u"proof_try",b"proof_try",u"signer",b"signer"]) -> None: ...
global___MsgChannelOpenAck = MsgChannelOpenAck

class MsgChannelOpenAckResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelOpenAckResponse = MsgChannelOpenAckResponse

class MsgChannelOpenConfirm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    PROOF_ACK_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    channel_id: typing.Text = ...
    proof_ack: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel_id : typing.Text = ...,
        proof_ack : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel_id",b"channel_id",u"port_id",b"port_id",u"proof_ack",b"proof_ack",u"proof_height",b"proof_height",u"signer",b"signer"]) -> None: ...
global___MsgChannelOpenConfirm = MsgChannelOpenConfirm

class MsgChannelOpenConfirmResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelOpenConfirmResponse = MsgChannelOpenConfirmResponse

class MsgChannelCloseInit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    channel_id: typing.Text = ...
    signer: typing.Text = ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel_id : typing.Text = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel_id",b"channel_id",u"port_id",b"port_id",u"signer",b"signer"]) -> None: ...
global___MsgChannelCloseInit = MsgChannelCloseInit

class MsgChannelCloseInitResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelCloseInitResponse = MsgChannelCloseInitResponse

class MsgChannelCloseConfirm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PORT_ID_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    PROOF_INIT_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    port_id: typing.Text = ...
    channel_id: typing.Text = ...
    proof_init: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        port_id : typing.Text = ...,
        channel_id : typing.Text = ...,
        proof_init : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channel_id",b"channel_id",u"port_id",b"port_id",u"proof_height",b"proof_height",u"proof_init",b"proof_init",u"signer",b"signer"]) -> None: ...
global___MsgChannelCloseConfirm = MsgChannelCloseConfirm

class MsgChannelCloseConfirmResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgChannelCloseConfirmResponse = MsgChannelCloseConfirmResponse

class MsgRecvPacket(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PACKET_FIELD_NUMBER: builtins.int
    PROOF_COMMITMENT_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    proof_commitment: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def packet(self) -> ibc.core.channel.v1.channel_pb2.Packet: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        packet : typing.Optional[ibc.core.channel.v1.channel_pb2.Packet] = ...,
        proof_commitment : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"packet",b"packet",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"packet",b"packet",u"proof_commitment",b"proof_commitment",u"proof_height",b"proof_height",u"signer",b"signer"]) -> None: ...
global___MsgRecvPacket = MsgRecvPacket

class MsgRecvPacketResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgRecvPacketResponse = MsgRecvPacketResponse

class MsgTimeout(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PACKET_FIELD_NUMBER: builtins.int
    PROOF_UNRECEIVED_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    NEXT_SEQUENCE_RECV_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    proof_unreceived: builtins.bytes = ...
    next_sequence_recv: builtins.int = ...
    signer: typing.Text = ...

    @property
    def packet(self) -> ibc.core.channel.v1.channel_pb2.Packet: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        packet : typing.Optional[ibc.core.channel.v1.channel_pb2.Packet] = ...,
        proof_unreceived : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        next_sequence_recv : builtins.int = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"packet",b"packet",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"next_sequence_recv",b"next_sequence_recv",u"packet",b"packet",u"proof_height",b"proof_height",u"proof_unreceived",b"proof_unreceived",u"signer",b"signer"]) -> None: ...
global___MsgTimeout = MsgTimeout

class MsgTimeoutResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgTimeoutResponse = MsgTimeoutResponse

class MsgTimeoutOnClose(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PACKET_FIELD_NUMBER: builtins.int
    PROOF_UNRECEIVED_FIELD_NUMBER: builtins.int
    PROOF_CLOSE_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    NEXT_SEQUENCE_RECV_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    proof_unreceived: builtins.bytes = ...
    proof_close: builtins.bytes = ...
    next_sequence_recv: builtins.int = ...
    signer: typing.Text = ...

    @property
    def packet(self) -> ibc.core.channel.v1.channel_pb2.Packet: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        packet : typing.Optional[ibc.core.channel.v1.channel_pb2.Packet] = ...,
        proof_unreceived : builtins.bytes = ...,
        proof_close : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        next_sequence_recv : builtins.int = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"packet",b"packet",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"next_sequence_recv",b"next_sequence_recv",u"packet",b"packet",u"proof_close",b"proof_close",u"proof_height",b"proof_height",u"proof_unreceived",b"proof_unreceived",u"signer",b"signer"]) -> None: ...
global___MsgTimeoutOnClose = MsgTimeoutOnClose

class MsgTimeoutOnCloseResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgTimeoutOnCloseResponse = MsgTimeoutOnCloseResponse

class MsgAcknowledgement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PACKET_FIELD_NUMBER: builtins.int
    ACKNOWLEDGEMENT_FIELD_NUMBER: builtins.int
    PROOF_ACKED_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    SIGNER_FIELD_NUMBER: builtins.int
    acknowledgement: builtins.bytes = ...
    proof_acked: builtins.bytes = ...
    signer: typing.Text = ...

    @property
    def packet(self) -> ibc.core.channel.v1.channel_pb2.Packet: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        packet : typing.Optional[ibc.core.channel.v1.channel_pb2.Packet] = ...,
        acknowledgement : builtins.bytes = ...,
        proof_acked : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        signer : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"packet",b"packet",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"acknowledgement",b"acknowledgement",u"packet",b"packet",u"proof_acked",b"proof_acked",u"proof_height",b"proof_height",u"signer",b"signer"]) -> None: ...
global___MsgAcknowledgement = MsgAcknowledgement

class MsgAcknowledgementResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___MsgAcknowledgementResponse = MsgAcknowledgementResponse



import grpc
import time

from typing import List, Optional

from .data import ReferencePrice, ReferencePriceUpdated
from .exceptions import NotFoundError, EmptyMsgError
from .proto.oracle.v1 import (
    query_pb2_grpc as oracle_query_grpc,
    query_pb2 as oracle_query,
    oracle_pb2 as oracle_type,
    tx_pb2_grpc as tx_oracle_grpc,
)
from .proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type
from .proto.cosmos.base.tendermint.v1beta1 import (
    query_pb2_grpc as tendermint_query_grpc,
    query_pb2 as tendermint_query,
)
from .proto.cosmos.auth.v1beta1 import (
    query_pb2_grpc as auth_query_grpc,
    query_pb2 as auth_query,
    auth_pb2 as auth_type,
)
from .proto.cosmos.tx.v1beta1 import (
    service_pb2_grpc as tx_service_grpc,
    service_pb2 as tx_service,
)


class Client:
    def __init__(self, grpc_endpoint: str):
        channel = grpc.insecure_channel(grpc_endpoint)
        self.stubOracle = oracle_query_grpc.QueryStub(channel)
        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(channel)
        self.stubAuth = auth_query_grpc.QueryStub(channel)
        self.stubTx = tx_service_grpc.ServiceStub(channel)
        self.stubOracleTx = tx_oracle_grpc.MsgStub(channel)

    def get_data_source(self, id: int) -> oracle_type.DataSource:
        return self.stubOracle.DataSource(oracle_query.QueryDataSourceRequest(data_source_id=id)).data_source

    def get_oracle_script(self, id: int) -> oracle_type.OracleScript:
        return self.stubOracle.OracleScript(oracle_query.QueryOracleScriptRequest(oracle_script_id=id)).oracle_script

    def get_request_by_id(self, id: int) -> oracle_query.QueryRequestResponse:
        return self.stubOracle.Request(oracle_query.QueryRequestRequest(request_id=id))

    def get_reporters(self, validator: str) -> oracle_type.ReportersPerValidator.reporters:
        return self.stubOracle.Reporters(oracle_query.QueryReportersRequest(validator_address=validator)).reporter

    def get_latest_block(self) -> tendermint_query.GetLatestBlockResponse:
        return self.stubCosmosTendermint.GetLatestBlock(tendermint_query.GetLatestBlockRequest())

    def get_account(self, address: str) -> Optional[auth_type.BaseAccount]:
        try:
            account_any = self.stubAuth.Account(auth_query.QueryAccountRequest(address=address)).account
            account = auth_type.BaseAccount()
            if account_any.Is(account.DESCRIPTOR):
                account_any.Unpack(account)
                return account
        except:
            return None

    def get_request_id_by_tx_hash(self, tx_hash: bytes) -> List[int]:
        tx = self.stubTx.GetTx(tx_service.GetTxRequest(hash=tx_hash))
        request_ids = []
        for tx in tx.tx_response.logs:
            request_event = [event for event in tx.events if event.type == "request" or event.type == "report"]
            if len(request_event) == 1:
                attrs = request_event[0].attributes
                attr_id = [attr for attr in attrs if attr.key == "id"]
                if len(attr_id) == 1:
                    request_id = attr_id[0].value
                    request_ids.append(int(request_id))
        if len(request_ids) == 0:
            raise NotFoundError("Request Id is not found")
        return request_ids

    def send_tx_sync_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        return self.stubTx.BroadcastTx(
            tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC)
        ).tx_response

    def send_tx_async_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        return self.stubTx.BroadcastTx(
            tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC)
        ).tx_response

    def send_tx_block_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        return self.stubTx.BroadcastTx(
            tx_service.BroadcastTxRequest(tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK)
        ).tx_response

    def get_chain_id(self) -> str:
        latest_block = self.get_latest_block()
        return latest_block.block.header.chain_id

    def get_reference_data(self, pairs: List[str], min_count: int, ask_count: int) -> List[ReferencePrice]:
        if len(pairs) == 0:
            raise EmptyMsgError("Pairs are required")

        symbols = set([symbol for pair in pairs for symbol in pair.split("/") if symbol != "USD"])
        price_data = self.stubOracle.RequestPrice(
            oracle_query.QueryRequestPriceRequest(symbols=symbols, min_count=min_count, ask_count=ask_count)
        )
        symbol_dict = {
            "USD": {
                "multiplier": 1000000000,
                "px": 1000000000,
                "resolve_time": int(time.time()),
            }
        }
        for price in price_data.price_results:
            symbol_dict[price.symbol] = {
                "multiplier": price.multiplier,
                "px": price.px,
                "resolve_time": price.resolve_time,
            }

        results = []
        for pair in pairs:
            [base_symbol, quote_symbol] = pair.split("/")
            results.append(
                ReferencePrice(
                    pair,
                    rate=(int(symbol_dict[base_symbol]["px"]) * int(symbol_dict[quote_symbol]["multiplier"]))
                    / (int(symbol_dict[quote_symbol]["px"]) * int(symbol_dict[base_symbol]["multiplier"])),
                    updated_at=ReferencePriceUpdated(
                        int(symbol_dict[base_symbol]["resolve_time"]),
                        int(symbol_dict[quote_symbol]["resolve_time"]),
                    ),
                )
            )
        return results

    def get_latest_request(
        self, oid: int, calldata: bytes, min_count: int, ask_count: int
    ) -> oracle_query.QueryRequestSearchResponse:
        return self.stubOracle.RequestSearch(
            oracle_query.QueryRequestSearchRequest(
                oracle_script_id=oid, calldata=calldata, min_count=min_count, ask_count=ask_count
            )
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Http(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    RULES_FIELD_NUMBER: builtins.int
    FULLY_DECODE_RESERVED_EXPANSION_FIELD_NUMBER: builtins.int
    fully_decode_reserved_expansion: builtins.bool = ...

    @property
    def rules(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HttpRule]: ...

    def __init__(self,
        *,
        rules : typing.Optional[typing.Iterable[global___HttpRule]] = ...,
        fully_decode_reserved_expansion : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"fully_decode_reserved_expansion",b"fully_decode_reserved_expansion",u"rules",b"rules"]) -> None: ...
global___Http = Http

class HttpRule(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    SELECTOR_FIELD_NUMBER: builtins.int
    GET_FIELD_NUMBER: builtins.int
    PUT_FIELD_NUMBER: builtins.int
    POST_FIELD_NUMBER: builtins.int
    DELETE_FIELD_NUMBER: builtins.int
    PATCH_FIELD_NUMBER: builtins.int
    CUSTOM_FIELD_NUMBER: builtins.int
    BODY_FIELD_NUMBER: builtins.int
    RESPONSE_BODY_FIELD_NUMBER: builtins.int
    ADDITIONAL_BINDINGS_FIELD_NUMBER: builtins.int
    selector: typing.Text = ...
    get: typing.Text = ...
    put: typing.Text = ...
    post: typing.Text = ...
    delete: typing.Text = ...
    patch: typing.Text = ...
    body: typing.Text = ...
    response_body: typing.Text = ...

    @property
    def custom(self) -> global___CustomHttpPattern: ...

    @property
    def additional_bindings(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HttpRule]: ...

    def __init__(self,
        *,
        selector : typing.Text = ...,
        get : typing.Text = ...,
        put : typing.Text = ...,
        post : typing.Text = ...,
        delete : typing.Text = ...,
        patch : typing.Text = ...,
        custom : typing.Optional[global___CustomHttpPattern] = ...,
        body : typing.Text = ...,
        response_body : typing.Text = ...,
        additional_bindings : typing.Optional[typing.Iterable[global___HttpRule]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"custom",b"custom",u"delete",b"delete",u"get",b"get",u"patch",b"patch",u"pattern",b"pattern",u"post",b"post",u"put",b"put"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"additional_bindings",b"additional_bindings",u"body",b"body",u"custom",b"custom",u"delete",b"delete",u"get",b"get",u"patch",b"patch",u"pattern",b"pattern",u"post",b"post",u"put",b"put",u"response_body",b"response_body",u"selector",b"selector"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"pattern",b"pattern"]) -> typing.Optional[typing_extensions.Literal["get","put","post","delete","patch","custom"]]: ...
global___HttpRule = HttpRule

class CustomHttpPattern(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    KIND_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    kind: typing.Text = ...
    path: typing.Text = ...

    def __init__(self,
        *,
        kind : typing.Text = ...,
        path : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"kind",b"kind",u"path",b"path"]) -> None: ...
global___CustomHttpPattern = CustomHttpPattern



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import google.protobuf.descriptor

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

interface_type: google.protobuf.descriptor.FieldDescriptor = ...

implements_interface: google.protobuf.descriptor.FieldDescriptor = ...

accepts_interface: google.protobuf.descriptor.FieldDescriptor = ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import tendermint.types.types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class BondStatus(metaclass=_BondStatus):
    V = typing.NewType('V', builtins.int)

global___BondStatus = BondStatus

BOND_STATUS_UNSPECIFIED = BondStatus.V(0)
BOND_STATUS_UNBONDED = BondStatus.V(1)
BOND_STATUS_UNBONDING = BondStatus.V(2)
BOND_STATUS_BONDED = BondStatus.V(3)

class _BondStatus(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BondStatus.V], builtins.type):  # type: ignore
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    BOND_STATUS_UNSPECIFIED = BondStatus.V(0)
    BOND_STATUS_UNBONDED = BondStatus.V(1)
    BOND_STATUS_UNBONDING = BondStatus.V(2)
    BOND_STATUS_BONDED = BondStatus.V(3)

class HistoricalInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HEADER_FIELD_NUMBER: builtins.int
    VALSET_FIELD_NUMBER: builtins.int

    @property
    def header(self) -> tendermint.types.types_pb2.Header: ...

    @property
    def valset(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Validator]: ...

    def __init__(self,
        *,
        header : typing.Optional[tendermint.types.types_pb2.Header] = ...,
        valset : typing.Optional[typing.Iterable[global___Validator]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"header",b"header"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"header",b"header",u"valset",b"valset"]) -> None: ...
global___HistoricalInfo = HistoricalInfo

class CommissionRates(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    RATE_FIELD_NUMBER: builtins.int
    MAX_RATE_FIELD_NUMBER: builtins.int
    MAX_CHANGE_RATE_FIELD_NUMBER: builtins.int
    rate: typing.Text = ...
    max_rate: typing.Text = ...
    max_change_rate: typing.Text = ...

    def __init__(self,
        *,
        rate : typing.Text = ...,
        max_rate : typing.Text = ...,
        max_change_rate : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"max_change_rate",b"max_change_rate",u"max_rate",b"max_rate",u"rate",b"rate"]) -> None: ...
global___CommissionRates = CommissionRates

class Commission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMMISSION_RATES_FIELD_NUMBER: builtins.int
    UPDATE_TIME_FIELD_NUMBER: builtins.int

    @property
    def commission_rates(self) -> global___CommissionRates: ...

    @property
    def update_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...

    def __init__(self,
        *,
        commission_rates : typing.Optional[global___CommissionRates] = ...,
        update_time : typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"commission_rates",b"commission_rates",u"update_time",b"update_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"commission_rates",b"commission_rates",u"update_time",b"update_time"]) -> None: ...
global___Commission = Commission

class Description(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MONIKER_FIELD_NUMBER: builtins.int
    IDENTITY_FIELD_NUMBER: builtins.int
    WEBSITE_FIELD_NUMBER: builtins.int
    SECURITY_CONTACT_FIELD_NUMBER: builtins.int
    DETAILS_FIELD_NUMBER: builtins.int
    moniker: typing.Text = ...
    identity: typing.Text = ...
    website: typing.Text = ...
    security_contact: typing.Text = ...
    details: typing.Text = ...

    def __init__(self,
        *,
        moniker : typing.Text = ...,
        identity : typing.Text = ...,
        website : typing.Text = ...,
        security_contact : typing.Text = ...,
        details : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"details",b"details",u"identity",b"identity",u"moniker",b"moniker",u"security_contact",b"security_contact",u"website",b"website"]) -> None: ...
global___Description = Description

class Validator(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    OPERATOR_ADDRESS_FIELD_NUMBER: builtins.int
    CONSENSUS_PUBKEY_FIELD_NUMBER: builtins.int
    JAILED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TOKENS_FIELD_NUMBER: builtins.int
    DELEGATOR_SHARES_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    UNBONDING_HEIGHT_FIELD_NUMBER: builtins.int
    UNBONDING_TIME_FIELD_NUMBER: builtins.int
    COMMISSION_FIELD_NUMBER: builtins.int
    MIN_SELF_DELEGATION_FIELD_NUMBER: builtins.int
    operator_address: typing.Text = ...
    jailed: builtins.bool = ...
    status: global___BondStatus.V = ...
    tokens: typing.Text = ...
    delegator_shares: typing.Text = ...
    unbonding_height: builtins.int = ...
    min_self_delegation: typing.Text = ...

    @property
    def consensus_pubkey(self) -> google.protobuf.any_pb2.Any: ...

    @property
    def description(self) -> global___Description: ...

    @property
    def unbonding_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...

    @property
    def commission(self) -> global___Commission: ...

    def __init__(self,
        *,
        operator_address : typing.Text = ...,
        consensus_pubkey : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        jailed : builtins.bool = ...,
        status : global___BondStatus.V = ...,
        tokens : typing.Text = ...,
        delegator_shares : typing.Text = ...,
        description : typing.Optional[global___Description] = ...,
        unbonding_height : builtins.int = ...,
        unbonding_time : typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        commission : typing.Optional[global___Commission] = ...,
        min_self_delegation : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"commission",b"commission",u"consensus_pubkey",b"consensus_pubkey",u"description",b"description",u"unbonding_time",b"unbonding_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"commission",b"commission",u"consensus_pubkey",b"consensus_pubkey",u"delegator_shares",b"delegator_shares",u"description",b"description",u"jailed",b"jailed",u"min_self_delegation",b"min_self_delegation",u"operator_address",b"operator_address",u"status",b"status",u"tokens",b"tokens",u"unbonding_height",b"unbonding_height",u"unbonding_time",b"unbonding_time"]) -> None: ...
global___Validator = Validator

class ValAddresses(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ADDRESSES_FIELD_NUMBER: builtins.int

    @property
    def addresses(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...

    def __init__(self,
        *,
        addresses : typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"addresses",b"addresses"]) -> None: ...
global___ValAddresses = ValAddresses

class DVPair(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_address: typing.Text = ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"validator_address",b"validator_address"]) -> None: ...
global___DVPair = DVPair

class DVPairs(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PAIRS_FIELD_NUMBER: builtins.int

    @property
    def pairs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DVPair]: ...

    def __init__(self,
        *,
        pairs : typing.Optional[typing.Iterable[global___DVPair]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pairs",b"pairs"]) -> None: ...
global___DVPairs = DVPairs

class DVVTriplet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_SRC_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_DST_ADDRESS_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_src_address: typing.Text = ...
    validator_dst_address: typing.Text = ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_src_address : typing.Text = ...,
        validator_dst_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"validator_dst_address",b"validator_dst_address",u"validator_src_address",b"validator_src_address"]) -> None: ...
global___DVVTriplet = DVVTriplet

class DVVTriplets(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TRIPLETS_FIELD_NUMBER: builtins.int

    @property
    def triplets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DVVTriplet]: ...

    def __init__(self,
        *,
        triplets : typing.Optional[typing.Iterable[global___DVVTriplet]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"triplets",b"triplets"]) -> None: ...
global___DVVTriplets = DVVTriplets

class Delegation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    SHARES_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_address: typing.Text = ...
    shares: typing.Text = ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_address : typing.Text = ...,
        shares : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"shares",b"shares",u"validator_address",b"validator_address"]) -> None: ...
global___Delegation = Delegation

class UnbondingDelegation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    ENTRIES_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_address: typing.Text = ...

    @property
    def entries(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UnbondingDelegationEntry]: ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_address : typing.Text = ...,
        entries : typing.Optional[typing.Iterable[global___UnbondingDelegationEntry]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"entries",b"entries",u"validator_address",b"validator_address"]) -> None: ...
global___UnbondingDelegation = UnbondingDelegation

class UnbondingDelegationEntry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CREATION_HEIGHT_FIELD_NUMBER: builtins.int
    COMPLETION_TIME_FIELD_NUMBER: builtins.int
    INITIAL_BALANCE_FIELD_NUMBER: builtins.int
    BALANCE_FIELD_NUMBER: builtins.int
    creation_height: builtins.int = ...
    initial_balance: typing.Text = ...
    balance: typing.Text = ...

    @property
    def completion_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...

    def __init__(self,
        *,
        creation_height : builtins.int = ...,
        completion_time : typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        initial_balance : typing.Text = ...,
        balance : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"completion_time",b"completion_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"balance",b"balance",u"completion_time",b"completion_time",u"creation_height",b"creation_height",u"initial_balance",b"initial_balance"]) -> None: ...
global___UnbondingDelegationEntry = UnbondingDelegationEntry

class RedelegationEntry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CREATION_HEIGHT_FIELD_NUMBER: builtins.int
    COMPLETION_TIME_FIELD_NUMBER: builtins.int
    INITIAL_BALANCE_FIELD_NUMBER: builtins.int
    SHARES_DST_FIELD_NUMBER: builtins.int
    creation_height: builtins.int = ...
    initial_balance: typing.Text = ...
    shares_dst: typing.Text = ...

    @property
    def completion_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...

    def __init__(self,
        *,
        creation_height : builtins.int = ...,
        completion_time : typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        initial_balance : typing.Text = ...,
        shares_dst : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"completion_time",b"completion_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"completion_time",b"completion_time",u"creation_height",b"creation_height",u"initial_balance",b"initial_balance",u"shares_dst",b"shares_dst"]) -> None: ...
global___RedelegationEntry = RedelegationEntry

class Redelegation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATOR_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_SRC_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_DST_ADDRESS_FIELD_NUMBER: builtins.int
    ENTRIES_FIELD_NUMBER: builtins.int
    delegator_address: typing.Text = ...
    validator_src_address: typing.Text = ...
    validator_dst_address: typing.Text = ...

    @property
    def entries(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RedelegationEntry]: ...

    def __init__(self,
        *,
        delegator_address : typing.Text = ...,
        validator_src_address : typing.Text = ...,
        validator_dst_address : typing.Text = ...,
        entries : typing.Optional[typing.Iterable[global___RedelegationEntry]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"delegator_address",b"delegator_address",u"entries",b"entries",u"validator_dst_address",b"validator_dst_address",u"validator_src_address",b"validator_src_address"]) -> None: ...
global___Redelegation = Redelegation

class Params(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    UNBONDING_TIME_FIELD_NUMBER: builtins.int
    MAX_VALIDATORS_FIELD_NUMBER: builtins.int
    MAX_ENTRIES_FIELD_NUMBER: builtins.int
    HISTORICAL_ENTRIES_FIELD_NUMBER: builtins.int
    BOND_DENOM_FIELD_NUMBER: builtins.int
    max_validators: builtins.int = ...
    max_entries: builtins.int = ...
    historical_entries: builtins.int = ...
    bond_denom: typing.Text = ...

    @property
    def unbonding_time(self) -> google.protobuf.duration_pb2.Duration: ...

    def __init__(self,
        *,
        unbonding_time : typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        max_validators : builtins.int = ...,
        max_entries : builtins.int = ...,
        historical_entries : builtins.int = ...,
        bond_denom : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"unbonding_time",b"unbonding_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"bond_denom",b"bond_denom",u"historical_entries",b"historical_entries",u"max_entries",b"max_entries",u"max_validators",b"max_validators",u"unbonding_time",b"unbonding_time"]) -> None: ...
global___Params = Params

class DelegationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DELEGATION_FIELD_NUMBER: builtins.int
    BALANCE_FIELD_NUMBER: builtins.int

    @property
    def delegation(self) -> global___Delegation: ...

    @property
    def balance(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...

    def __init__(self,
        *,
        delegation : typing.Optional[global___Delegation] = ...,
        balance : typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"balance",b"balance",u"delegation",b"delegation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"balance",b"balance",u"delegation",b"delegation"]) -> None: ...
global___DelegationResponse = DelegationResponse

class RedelegationEntryResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    REDELEGATION_ENTRY_FIELD_NUMBER: builtins.int
    BALANCE_FIELD_NUMBER: builtins.int
    balance: typing.Text = ...

    @property
    def redelegation_entry(self) -> global___RedelegationEntry: ...

    def __init__(self,
        *,
        redelegation_entry : typing.Optional[global___RedelegationEntry] = ...,
        balance : typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"redelegation_entry",b"redelegation_entry"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"balance",b"balance",u"redelegation_entry",b"redelegation_entry"]) -> None: ...
global___RedelegationEntryResponse = RedelegationEntryResponse

class RedelegationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    REDELEGATION_FIELD_NUMBER: builtins.int
    ENTRIES_FIELD_NUMBER: builtins.int

    @property
    def redelegation(self) -> global___Redelegation: ...

    @property
    def entries(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RedelegationEntryResponse]: ...

    def __init__(self,
        *,
        redelegation : typing.Optional[global___Redelegation] = ...,
        entries : typing.Optional[typing.Iterable[global___RedelegationEntryResponse]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"redelegation",b"redelegation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"entries",b"entries",u"redelegation",b"redelegation"]) -> None: ...
global___RedelegationResponse = RedelegationResponse

class Pool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NOT_BONDED_TOKENS_FIELD_NUMBER: builtins.int
    BONDED_TOKENS_FIELD_NUMBER: builtins.int
    not_bonded_tokens: typing.Text = ...
    bonded_tokens: typing.Text = ...

    def __init__(self,
        *,
        not_bonded_tokens : typing.Text = ...,
        bonded_tokens : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"bonded_tokens",b"bonded_tokens",u"not_bonded_tokens",b"not_bonded_tokens"]) -> None: ...
global___Pool = Pool



from typing import Any, Tuple
from .exceptions import SchemaError, DecodeError


class PyObiSpec(object):
    impls = []

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.impls.append(cls)

    @classmethod
    def from_spec(cls, spec: str) -> Any:
        for impl in cls.impls:
            if impl.match_schema(spec):
                return impl(spec)
        raise SchemaError("Cannot parse spec: {}".format(spec))

    def __init__(self, spec):
        raise NotImplementedError()

    @classmethod
    def match_schema(cls, schema):
        raise NotImplementedError()

    def encode(self, value):
        raise NotImplementedError()

    def decode(self, data):
        raise NotImplementedError()


class PyObiInteger(PyObiSpec):
    def __init__(self, spec):
        self.is_signed = spec[0] == "i"
        self.size_in_bytes = int(spec[1:]) // 8

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema[:1] in ["i", "u"] and schema[1:] in ["8", "16", "32", "64", "128", "256"]

    def encode(self, value: int) -> bytes:
        return value.to_bytes(self.size_in_bytes, byteorder="big", signed=self.is_signed)

    def decode(self, data: bytes) -> Tuple[int, bytes]:
        return (
            int.from_bytes(data[: self.size_in_bytes], byteorder="big", signed=self.is_signed),
            data[self.size_in_bytes :],
        )


class PyObiBool(PyObiSpec):
    def __init__(self, spec="bool"):
        pass

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema == "bool"

    def encode(self, value: bool) -> bytes:
        return PyObiInteger("u8").encode(1 if value else 0)

    def decode(self, data: bytes) -> Tuple[bool, bytes]:
        u8, remaining = PyObiInteger("u8").decode(data)
        if u8 == 1:
            return True, remaining
        elif u8 == 0:
            return False, remaining
        raise ValueError("Boolean value must be 1 or 0 but got {}".format(u8))


class PyObiArray(PyObiSpec):
    def __init__(self, _spec):
        [spec, size] = _spec[1:-1].rsplit(";", 1)
        self.size = int(size, 10)
        self.intl_obi = self.from_spec(spec)

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        if not (schema[0] == "[" and schema[-1] == "]"):
            return False
        try:
            [spec, size] = schema[1:-1].rsplit(";", 1)
        except:
            return False

        if not size.isdigit():
            return False

        for impl in cls.impls:
            if impl.match_schema(spec):
                return True

        return False

    def encode(self, value: list) -> bytes:
        if len(value) != self.size:
            raise ValueError("array size should be {} but got {}".format(self.size, len(value)))
        result = b""
        for each in value:
            result = result + self.intl_obi.encode(each)
        return result

    def decode(self, data: bytes) -> Tuple[list, bytes]:
        remaining = data[:]
        result = []
        for _ in range(self.size):
            each, remaining = self.intl_obi.decode(remaining)
            result.append(each)
        return result, remaining


class PyObiVector(PyObiSpec):
    def __init__(self, spec):
        self.intl_obi = self.from_spec(spec[1:-1])

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema[0] == "[" and schema[-1] == "]"

    def encode(self, value: list) -> bytes:
        result = PyObiInteger("u32").encode(len(value))
        for each in value:
            result = result + self.intl_obi.encode(each)
        return result

    def decode(self, data: bytes) -> Tuple[list, bytes]:
        length, remaining = PyObiInteger("u32").decode(data)
        result = []
        for _ in range(length):
            each, remaining = self.intl_obi.decode(remaining)
            result.append(each)
        return result, remaining


class PyObiStruct(PyObiSpec):
    def __init__(self, spec):
        self.intl_obi_kvs = []
        fields = [""]
        curly_count = 0
        for c in spec[1:-1]:
            if c == "," and curly_count == 0:
                fields.append("")
            else:
                fields[-1] = fields[-1] + c
                if c == "{":
                    curly_count += 1
                if c == "}":
                    curly_count -= 1
        for each in fields:
            tokens = each.split(":", 1)
            if len(tokens) != 2:
                raise ValueError("Expect at least one colon for each struct field")
            self.intl_obi_kvs.append((tokens[0], self.from_spec(tokens[1])))

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema[0] == "{" and schema[-1] == "}"

    def encode(self, value: dict) -> bytes:
        result = b""
        for key, spec in self.intl_obi_kvs:
            result = result + spec.encode(value[key])
        return result

    def decode(self, data: bytes) -> Tuple[dict, bytes]:
        result = {}
        for key, spec in self.intl_obi_kvs:
            result[key], data = spec.decode(data)
        return result, data


class PyObiString(PyObiSpec):
    def __init__(self, spec="string"):
        pass

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema == "string"

    def encode(self, value: str) -> bytes:
        return PyObiInteger("u32").encode(len(value)) + value.encode()

    def decode(self, data: bytes) -> Tuple[str, bytes]:
        length, remaining = PyObiInteger("u32").decode(data)
        return remaining[:length].decode(), remaining[length:]


class PyObiBytes(PyObiSpec):
    def __init__(self, spec="bytes"):
        pass

    @classmethod
    def match_schema(cls, schema: str) -> bool:
        return schema == "bytes"

    def encode(self, value: bytes) -> bytes:
        return PyObiInteger("u32").encode(len(value)) + value

    def decode(self, data: bytes) -> Tuple[bytes, bytes]:
        length, remaining = PyObiInteger("u32").decode(data)
        return remaining[:length], remaining[length:]


class PyObi(object):
    def __init__(self, schema):
        normalized_schema = "".join(schema.split())
        tokens = normalized_schema.split("/")
        self.schemas = [PyObiSpec.from_spec(token) for token in tokens]

    def encode(self, data: Any, index=0) -> bytes:
        return self.schemas[index].encode(data)

    def decode(self, data: bytes, index=0) -> Any:
        result, remaining = self.schemas[index].decode(data)
        if remaining:
            raise DecodeError("Not all data is consumed after decoding input")
        return result

    def encode_input(self, data: Any) -> bytes:
        return self.encode(data, index=0)

    def encode_output(self, data: Any) -> bytes:
        return self.encode(data, index=1)

    def decode_input(self, data: bytes) -> Any:
        return self.decode(data, index=0)

    def decode_output(self, data: bytes) -> Any:
        return self.decode(data, index=1)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class NetAddress(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    ip: typing.Text = ...
    port: builtins.int = ...

    def __init__(self,
        *,
        id : typing.Text = ...,
        ip : typing.Text = ...,
        port : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"id",b"id",u"ip",b"ip",u"port",b"port"]) -> None: ...
global___NetAddress = NetAddress

class ProtocolVersion(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    P2P_FIELD_NUMBER: builtins.int
    BLOCK_FIELD_NUMBER: builtins.int
    APP_FIELD_NUMBER: builtins.int
    p2p: builtins.int = ...
    block: builtins.int = ...
    app: builtins.int = ...

    def __init__(self,
        *,
        p2p : builtins.int = ...,
        block : builtins.int = ...,
        app : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"app",b"app",u"block",b"block",u"p2p",b"p2p"]) -> None: ...
global___ProtocolVersion = ProtocolVersion

class DefaultNodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PROTOCOL_VERSION_FIELD_NUMBER: builtins.int
    DEFAULT_NODE_ID_FIELD_NUMBER: builtins.int
    LISTEN_ADDR_FIELD_NUMBER: builtins.int
    NETWORK_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    CHANNELS_FIELD_NUMBER: builtins.int
    MONIKER_FIELD_NUMBER: builtins.int
    OTHER_FIELD_NUMBER: builtins.int
    default_node_id: typing.Text = ...
    listen_addr: typing.Text = ...
    network: typing.Text = ...
    version: typing.Text = ...
    channels: builtins.bytes = ...
    moniker: typing.Text = ...

    @property
    def protocol_version(self) -> global___ProtocolVersion: ...

    @property
    def other(self) -> global___DefaultNodeInfoOther: ...

    def __init__(self,
        *,
        protocol_version : typing.Optional[global___ProtocolVersion] = ...,
        default_node_id : typing.Text = ...,
        listen_addr : typing.Text = ...,
        network : typing.Text = ...,
        version : typing.Text = ...,
        channels : builtins.bytes = ...,
        moniker : typing.Text = ...,
        other : typing.Optional[global___DefaultNodeInfoOther] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"other",b"other",u"protocol_version",b"protocol_version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"channels",b"channels",u"default_node_id",b"default_node_id",u"listen_addr",b"listen_addr",u"moniker",b"moniker",u"network",b"network",u"other",b"other",u"protocol_version",b"protocol_version",u"version",b"version"]) -> None: ...
global___DefaultNodeInfo = DefaultNodeInfo

class DefaultNodeInfoOther(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TX_INDEX_FIELD_NUMBER: builtins.int
    RPC_ADDRESS_FIELD_NUMBER: builtins.int
    tx_index: typing.Text = ...
    rpc_address: typing.Text = ...

    def __init__(self,
        *,
        tx_index : typing.Text = ...,
        rpc_address : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"rpc_address",b"rpc_address",u"tx_index",b"tx_index"]) -> None: ...
global___DefaultNodeInfoOther = DefaultNodeInfoOther



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class MsgSubmitEvidence(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    SUBMITTER_FIELD_NUMBER: builtins.int
    EVIDENCE_FIELD_NUMBER: builtins.int
    submitter: typing.Text = ...

    @property
    def evidence(self) -> google.protobuf.any_pb2.Any: ...

    def __init__(self,
        *,
        submitter : typing.Text = ...,
        evidence : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"evidence",b"evidence"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"evidence",b"evidence",u"submitter",b"submitter"]) -> None: ...
global___MsgSubmitEvidence = MsgSubmitEvidence

class MsgSubmitEvidenceResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HASH_FIELD_NUMBER: builtins.int
    hash: builtins.bytes = ...

    def __init__(self,
        *,
        hash : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"hash",b"hash"]) -> None: ...
global___MsgSubmitEvidenceResponse = MsgSubmitEvidenceResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.upgrade.v1beta1.upgrade_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class QueryCurrentPlanRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___QueryCurrentPlanRequest = QueryCurrentPlanRequest

class QueryCurrentPlanResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PLAN_FIELD_NUMBER: builtins.int

    @property
    def plan(self) -> cosmos.upgrade.v1beta1.upgrade_pb2.Plan: ...

    def __init__(self,
        *,
        plan : typing.Optional[cosmos.upgrade.v1beta1.upgrade_pb2.Plan] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"plan",b"plan"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"plan",b"plan"]) -> None: ...
global___QueryCurrentPlanResponse = QueryCurrentPlanResponse

class QueryAppliedPlanRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NAME_FIELD_NUMBER: builtins.int
    name: typing.Text = ...

    def __init__(self,
        *,
        name : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"name",b"name"]) -> None: ...
global___QueryAppliedPlanRequest = QueryAppliedPlanRequest

class QueryAppliedPlanResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int = ...

    def __init__(self,
        *,
        height : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"height",b"height"]) -> None: ...
global___QueryAppliedPlanResponse = QueryAppliedPlanResponse

class QueryUpgradedConsensusStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LAST_HEIGHT_FIELD_NUMBER: builtins.int
    last_height: builtins.int = ...

    def __init__(self,
        *,
        last_height : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"last_height",b"last_height"]) -> None: ...
global___QueryUpgradedConsensusStateRequest = QueryUpgradedConsensusStateRequest

class QueryUpgradedConsensusStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    UPGRADED_CONSENSUS_STATE_FIELD_NUMBER: builtins.int

    @property
    def upgraded_consensus_state(self) -> google.protobuf.any_pb2.Any: ...

    def __init__(self,
        *,
        upgraded_consensus_state : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"upgraded_consensus_state",b"upgraded_consensus_state"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"upgraded_consensus_state",b"upgraded_consensus_state"]) -> None: ...
global___QueryUpgradedConsensusStateResponse = QueryUpgradedConsensusStateResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import ibc.core.client.v1.client_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class QueryClientStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CLIENT_ID_FIELD_NUMBER: builtins.int
    client_id: typing.Text = ...

    def __init__(self,
        *,
        client_id : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"client_id",b"client_id"]) -> None: ...
global___QueryClientStateRequest = QueryClientStateRequest

class QueryClientStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CLIENT_STATE_FIELD_NUMBER: builtins.int
    PROOF_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    proof: builtins.bytes = ...

    @property
    def client_state(self) -> google.protobuf.any_pb2.Any: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        client_state : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        proof : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"client_state",b"client_state",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"client_state",b"client_state",u"proof",b"proof",u"proof_height",b"proof_height"]) -> None: ...
global___QueryClientStateResponse = QueryClientStateResponse

class QueryClientStatesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> None: ...
global___QueryClientStatesRequest = QueryClientStatesRequest

class QueryClientStatesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CLIENT_STATES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def client_states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.client.v1.client_pb2.IdentifiedClientState]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        client_states : typing.Optional[typing.Iterable[ibc.core.client.v1.client_pb2.IdentifiedClientState]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"client_states",b"client_states",u"pagination",b"pagination"]) -> None: ...
global___QueryClientStatesResponse = QueryClientStatesResponse

class QueryConsensusStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CLIENT_ID_FIELD_NUMBER: builtins.int
    REVISION_NUMBER_FIELD_NUMBER: builtins.int
    REVISION_HEIGHT_FIELD_NUMBER: builtins.int
    LATEST_HEIGHT_FIELD_NUMBER: builtins.int
    client_id: typing.Text = ...
    revision_number: builtins.int = ...
    revision_height: builtins.int = ...
    latest_height: builtins.bool = ...

    def __init__(self,
        *,
        client_id : typing.Text = ...,
        revision_number : builtins.int = ...,
        revision_height : builtins.int = ...,
        latest_height : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"client_id",b"client_id",u"latest_height",b"latest_height",u"revision_height",b"revision_height",u"revision_number",b"revision_number"]) -> None: ...
global___QueryConsensusStateRequest = QueryConsensusStateRequest

class QueryConsensusStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CONSENSUS_STATE_FIELD_NUMBER: builtins.int
    PROOF_FIELD_NUMBER: builtins.int
    PROOF_HEIGHT_FIELD_NUMBER: builtins.int
    proof: builtins.bytes = ...

    @property
    def consensus_state(self) -> google.protobuf.any_pb2.Any: ...

    @property
    def proof_height(self) -> ibc.core.client.v1.client_pb2.Height: ...

    def __init__(self,
        *,
        consensus_state : typing.Optional[google.protobuf.any_pb2.Any] = ...,
        proof : builtins.bytes = ...,
        proof_height : typing.Optional[ibc.core.client.v1.client_pb2.Height] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"consensus_state",b"consensus_state",u"proof_height",b"proof_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"consensus_state",b"consensus_state",u"proof",b"proof",u"proof_height",b"proof_height"]) -> None: ...
global___QueryConsensusStateResponse = QueryConsensusStateResponse

class QueryConsensusStatesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CLIENT_ID_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    client_id: typing.Text = ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...

    def __init__(self,
        *,
        client_id : typing.Text = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"client_id",b"client_id",u"pagination",b"pagination"]) -> None: ...
global___QueryConsensusStatesRequest = QueryConsensusStatesRequest

class QueryConsensusStatesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CONSENSUS_STATES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int

    @property
    def consensus_states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[ibc.core.client.v1.client_pb2.ConsensusStateWithHeight]: ...

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...

    def __init__(self,
        *,
        consensus_states : typing.Optional[typing.Iterable[ibc.core.client.v1.client_pb2.ConsensusStateWithHeight]] = ...,
        pagination : typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"consensus_states",b"consensus_states",u"pagination",b"pagination"]) -> None: ...
global___QueryConsensusStatesResponse = QueryConsensusStatesResponse

class QueryClientParamsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    def __init__(self,
        ) -> None: ...
global___QueryClientParamsRequest = QueryClientParamsRequest

class QueryClientParamsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PARAMS_FIELD_NUMBER: builtins.int

    @property
    def params(self) -> ibc.core.client.v1.client_pb2.Params: ...

    def __init__(self,
        *,
        params : typing.Optional[ibc.core.client.v1.client_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"params",b"params"]) -> None: ...
global___QueryClientParamsResponse = QueryClientParamsResponse
